# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,proDAFit)
export(abundances)
export(coefficient_variance_matrices)
export(coefficients)
export(convergence)
export(dist_approx)
export(feature_parameters)
export(generate_synthetic_data)
export(hyper_parameters)
export(invprobit)
export(median_normalization)
export(pd_lm)
export(pd_row_f_test)
export(pd_row_t_test)
export(proDA)
export(reference_level)
export(result_names)
export(test_diff)
exportClasses(proDAFit)
exportMethods(abundances)
exportMethods(coefficient_variance_matrices)
exportMethods(coefficients)
exportMethods(convergence)
exportMethods(design)
exportMethods(dist_approx)
exportMethods(feature_parameters)
exportMethods(hyper_parameters)
exportMethods(predict)
exportMethods(reference_level)
exportMethods(result_names)
import(methods)
import(stats)
importFrom(BiocGenerics,design)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"mcols<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,cbind)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,mcols)
importFrom(SummarizedExperiment,rbind)
importFrom(SummarizedExperiment,rowData)
importFrom(utils,.DollarNames)
importFrom(utils,read.delim)
