% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{proDAFit-class}
\alias{proDAFit-class}
\alias{.proDAFit}
\title{proDA Class Definition}
\description{
proDA Class Definition
}
